#!/bin/sh
#\
exec wish $0 $@

#------------------------------------------------
#
# setup.tcl --
#    This is a Tcl/Tk script which helps install TASH.
#    It collects information about the environment
#    and creates a file, Makefile.macros.  This file
#    is included in Makefiles to customize.
#
# Copyright (c) 1997 Terry J. Westley
#
# See the file "license.htm" for information on usage and
# redistribution of this file, and  a DISCLAIMER OF ALL WARRANTIES.
#
#------------------------------------------------

proc cequal {left right} {
   return [expr [string compare $left $right] == 0]
}

proc lempty {string} {
   return [expr [string length $string] == 0]
}

proc setvar {name value comments} {
   global tashvar tashorder tashcomments
   set tashvar($name) $value
   set tashcomments($name) $comments
   lappend tashorder $name
}

# Create the Makefile macros file
#--------------------------------
proc Save {makefile} {
   global tashorder tashvar tashcomments
   if [catch {open $makefile w} makefid] {
      puts stderr $makefid
      exit
   }
   puts $makefid "#$makefile"
   puts $makefid \
{#####################################################
#
# This file, Makefile.macros, contains macros used to
# customize Makefiles in various TASH directories.
#
# It is automatically generated by the setup.tcl script.
# To change them, you may either edit them directly
# with a text editor or run setup.tcl again.
#
#####################################################}

   foreach name $tashorder {
      foreach line [split $tashcomments($name) "\n"] {
	 puts $makefid [string trimleft $line]
      }
      puts $makefid [format "%-18s = %s" $name $tashvar($name)]
   }
   catch {close $makefid}
}

proc fileDialog {w ent title initial} {
    set types {
	{"All files" *}
    }
    set file [tk_getOpenFile -filetypes $types -parent $w \
       -initialdir $initial -title $title]
    if [string compare $file ""] {
	$ent delete 0 end
	$ent insert 0 $file
	$ent xview end
    }
}

# Establish values for all macros depending on platform
#------------------------------------------------------
proc Set_Macros {platform os osVersion} {
   global tcl_version tk_version tcl_interactive tcl_library tk_library

   set tcl_link_switches "\$(TCL_LIB) ../\$(PLATFORM)/tclmacro.o"
   set tk_link_switches  "\$(TK_LIB)  ../\$(PLATFORM)/tkmacro.o"
   set x11_lib           ""
   set x11_includes      ""
   set tcl_lib           ""
   set tk_lib            ""
   set tcl_includes      [file join \
      [file dirname [file dirname $tcl_library]] include]

   switch $platform {
      "windows" {
	 if [cequal $os "Windows 95"] {
	    set platform "win95"
	 }
	 regsub {\.} $tcl_version {} tcl_short_version
	 set tcl_lib "../\$(PLATFORM)/libtcl${tcl_short_version}.a"
	 set tclsh "tclsh${tcl_short_version}"
	 regsub {\.} $tk_version {} tk_short_version
	 set tk_lib "../\$(PLATFORM)/libtk${tk_short_version}.a"
      }
      "unix" {
	 set tclsh "tclsh"
	 append tcl_link_switches " -lm -ldl"
	 if [cequal $os "SunOS"] {
	    append tcl_link_switches " -lsocket -lnsl"
	 }
	 append tk_link_switches " -L\$(X11_LIB) -lX11"
	 foreach dir [list /usr/openwin /usr/X /usr/X11R6 /usr] {
	    set dir [file join $dir lib]
	    foreach file [list "libX11.so" "libX11.a"] {
	       if [file exists [file join $dir $file]] {
		  set x11_lib $dir
		  break
	       }
	    }
	 }
	 foreach dir [list /usr/openwin /usr/X /usr/X11R6 /usr] {
	    set file [file join $dir include]
	    if [file exists $file] {
	       set x11_includes $file
	       break
	    }
	 }
	 set tcl_lib [file join [file dirname $tcl_library] libtcl$tcl_version.a]
	 set tk_lib  [file join [file dirname $tk_library]  libtk$tk_version.a]
      }
   }

   setvar PLATFORM          $platform {
      # Directory that contains the compiled TASH library code}
   setvar OS                $os {
      # Operating system}
   setvar OSVERSION         $osVersion {
      # Operating system version}
   setvar TASH_VERSION      "8.0.2" {
      # TASH version}
   setvar TCLSH             $tclsh {
      # Tclsh executable}
   setvar SRC_DIR           "../src" {
      # TASH source directory}
   setvar ADA_LIBS          "-I\$(SRC_DIR) -I../\$(PLATFORM)" {
      # Ada library directories}
   if [lempty $x11_lib] {
      setvar X11_LIB        "" {
	 # X11 library directory}
   } else {
      setvar X11_LIB        "$x11_lib" {
	 # X11 library directory}
   }
   if [lempty $x11_includes] {
      setvar X11_INCLUDES   "" {
	 # X11 includes directory}
   } else {
      setvar X11_INCLUDES   "-I$x11_includes" {
	 # X11 includes directory}
   }
   setvar TCL_INCLUDES   "-I$tcl_includes" {
      # TCL includes directory}
   setvar TCL_VERSION       "$tcl_version" {
      # Tcl version}
   setvar TCL_LIB           "$tcl_lib" {
      # File name of Tcl library}
   setvar TCL_LINK_SWITCHES "$tcl_link_switches" {
      # Tcl link switches}
   setvar TK_VERSION        "$tk_version" {
      # Tk version}
   setvar TK_LIB            "$tk_lib" {
      # File name of Tk library}
   setvar TK_LINK_SWITCHES  "$tk_link_switches" {
      # Tk link switches}
}

# Establish values for all macros depending on platform
#------------------------------------------------------
Set_Macros $tcl_platform(platform) $tcl_platform(os) $tcl_platform(osVersion)

# Create window for installer to review and edit macro values
#------------------------------------------------------------
set makefile [file join [file dirname $argv0] Makefile.macros]
wm title    . "TASH $tashvar(TASH_VERSION) Setup -- $makefile"
wm iconname . "TASH Setup"

message .instructions -justify left -aspect 500 -pady 20 -padx 20 -text \
"Following are a set of macros used to customize Makefiles\
in various TASH directories.  Please make any necessary corrections,\
then press \"Save\" to save into the file, $makefile."

pack .instructions -side top -fill x -expand yes

set g [frame .grid]
pack $g -side top

set row 0

foreach name $tashorder {
   set w [string tolower $name]
   label $g.$w-label -text "$name: " -anchor e
   grid $g.$w-label  -row $row -column 0 -sticky e
   entry $g.$w-entry -width 30
   $g.$w-entry insert end $tashvar($name)
   switch -regexp $name {
      VERSION {
	 $g.$w-entry configure -state disabled
      }
   }
   grid $g.$w-entry  -row $row -column 1
   if 0 {
      if [regexp {^(/|\.)} $tashvar($name)] {
	 button $g.$w-browse -text "Browse" \
	    -command "fileDialog $g $g.$w-entry $name $tashvar($name)"
	 grid $g.$w-browse -row $row -column 2
      }
   }
   incr row
}

frame .buttons
pack .buttons -side bottom -fill x -pady 2m
button .buttons.save -text Save -command "Save \"$makefile\";exit"
button .buttons.cancel -text "Cancel" -command "exit"
if $tcl_interactive {
   button .buttons.test -text "Test" -command "Update_Macros $g"
   pack .buttons.test .buttons.save .buttons.cancel -side left -expand 1
} else {
   pack .buttons.save .buttons.cancel -side left -expand 1
}

proc Update_Macros {g} {
   global tashorder tashvar
   set platform  [$g.platform-entry get]
   set os        [$g.os-entry get]
   set osVersion [$g.osversion-entry get]
   Set_Macros $platform $os $osVersion
   set row 0
   foreach name $tashorder {
      set w [string tolower $name]
      $g.$w-entry configure -state normal
      $g.$w-entry delete 0 end
      $g.$w-entry insert end $tashvar($name)
      switch -regexp $name {
	 VERSION {
	    $g.$w-entry configure -state disabled
	 }
      }
      incr row
   }
}
